"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.frontmatterToMDX = void 0;
const fs_1 = require("fs");
const path = require("path");
const yaml = require("yaml");
const utils_1 = require("./utils");
/**
 * Parses the YAML frontmatter and converts it to an MDX default export
 */
function frontmatterToMDX(options) {
    return async (root, file) => {
        const stat = await fs_1.promises.stat(file.path);
        const frontmatter = extractFrontMatter(root);
        const insertionPoint = findInsertionPoint(root);
        const layoutPath = getLayoutPath(frontmatter, file, options);
        // Serialize the props as React attributes
        const props = toReactElementAttributes({
            ...frontmatter,
            createdAt: stat.birthtime,
            modifiedAt: stat.mtime
        });
        root.children.splice(insertionPoint, 0, {
            type: "import",
            value: `import Layout from ${JSON.stringify(layoutPath)};\n`,
        }, ...Object.entries(frontmatter).map(([key, value]) => ({
            type: "export",
            value: `export const ${key} = ${JSON.stringify(value)};\n`,
        })), {
            type: "export",
            value: `export const createdAt = new Date(${stat.birthtimeMs})\n`,
        }, {
            type: "export",
            value: `export const modifiedAt = new Date(${stat.mtimeMs})\n`,
        }, {
            type: "export",
            default: true,
            value: `export default (props) => <Layout${props}\n  {...props}\n/>\n`,
        });
        return root;
    };
}
exports.frontmatterToMDX = frontmatterToMDX;
/**
 * Removes the frontmatter node, parses it, and returns the parsed value
 */
function extractFrontMatter(tree) {
    const frontmatterIndex = tree.children.findIndex(node => node.type === "yaml");
    if (frontmatterIndex >= 0) {
        const [frontmatterNode] = tree.children.splice(frontmatterIndex, 1);
        const frontmatter = yaml.parse(frontmatterNode.value);
        if (typeof frontmatter === "object" && frontmatter !== null) {
            return frontmatter;
        }
    }
    return {};
}
/**
 * Finds the point where we should insert the MDX default export.
 * This is after all "import" nodes, but before any "JSX" or "markdown" nodes.
 */
function findInsertionPoint(tree) {
    for (let i = 0; i < tree.children.length; i++) {
        const node = tree.children[i];
        switch (node.type) {
            case "import":
                continue;
            default:
                return i;
        }
    }
    return 0;
}
/**
 * Returns the path of layout component to use
 */
function getLayoutPath(frontmatter, file, options) {
    const layout = String(frontmatter.layout || options.defaultLayout);
    const layoutPath = path.relative(file.dirname, path.join(options.layoutsDir, layout));
    delete frontmatter.layout;
    // "import" statements should use POSIX separators
    return utils_1.toPosixPath(layoutPath);
}
/**
 * Serializes the given frontmatter as React element attributes
 */
function toReactElementAttributes(obj) {
    let props = "";
    // eslint-disable-next-line prefer-const
    for (let [key, value] of Object.entries(obj)) {
        if (typeof value === "string") {
            value = JSON.stringify(value);
        }
        else if (value instanceof Date) {
            value = `{new Date(${value.getTime()})}`;
        }
        else if (Array.isArray(value)) {
            value = `{[\n    ${value.map(item => JSON.stringify(item)).join(",\n    ")}\n  ]}`;
        }
        else {
            value = `{${JSON.stringify(value)}}`;
        }
        props += `\n  ${key}=${value}`;
    }
    return props;
}
//# sourceMappingURL=frontmatter-to-mdx.js.map