"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.linkToMDX = void 0;
const fs_1 = require("fs");
const path = require("path");
const url_1 = require("url");
const utils_1 = require("./utils");
const mdxExtension = ".mdx";
const mdxIndex = "index.mdx";
/**
 * Rewrites links to MDX files (e.g. `../other-page.mdx` becomes `../other-page`).
 * Also verifies that the target file exists.
 */
function linkToMDX(options) {
    return async (root, file) => {
        await crawl(root, file, options);
        return root;
    };
}
exports.linkToMDX = linkToMDX;
/**
 * Recursively rewrites links to MDX files
 */
async function crawl(tree, file, options) {
    const promises = [];
    for (const node of tree.children) {
        switch (node.type) {
            case "link":
                if (typeof node.url === "string" && node.url.includes(mdxExtension)) {
                    const newLink = await validateAndRewriteLink(node.url, file, options);
                    if (newLink) {
                        node.url = newLink;
                    }
                }
                break;
            case "mdxBlockElement":
            case "mdxSpanElement":
                const mdxNode = node;
                for (const attribute of mdxNode.attributes) {
                    if (typeof attribute.value === "string" && attribute.value.includes(mdxExtension)) {
                        const newLink = await validateAndRewriteLink(attribute.value, file, options);
                        if (newLink) {
                            attribute.value = newLink;
                        }
                    }
                }
                break;
        }
        if (node.children) {
            const promise = crawl(node, file, options);
            promises.push(promise);
        }
    }
    // Process all child nodes simultaneously
    await Promise.all(promises);
}
/**
 * Determines whether the link points to a local MDX file. If it does, then the link is validated
 * and rewritten to be relative to the Next.js "Pages" directory (which is required by Next.js).
 *
 * @param link - An absolute or relative URL that may point to an MDX file
 * @param file - The file that contains the link
 *
 * @returns
 * If `link` points to a local MDX file, then a new, rewritten link is returned.
 * Otherwise, `undefined` is returned.
 */
async function validateAndRewriteLink(link, file, options) {
    // Resolve the link, relative to the current file
    const fileURL = url_1.pathToFileURL(file.path);
    const linkURL = new URL(link, fileURL);
    // Abort if the link does not point to a local MDX file
    if (linkURL.protocol !== "file:" || !linkURL.pathname.endsWith(mdxExtension)) {
        return;
    }
    const absolutePath = url_1.fileURLToPath(linkURL);
    await validateLink(absolutePath, link, file, options);
    return rewriteLink(absolutePath, linkURL, options);
}
/**
 * Verifies that a link points to a valid file
 *
 * @param absolutePath - The absolute file path to validate
 * @param link - The original link, as authored in the file
 * @param file - The file that contains the link
 * @param options - An options object
 *
 * @throws A URIError if the target file does not exist
 */
async function validateLink(absolutePath, link, file, options) {
    const { fileCache } = options;
    // See if we've already started checking this file
    const cached = await fileCache.get(absolutePath);
    if (cached.exists && cached.expires >= Date.now()) {
        // This file has been verified, and the verification is not expired
        return;
    }
    // This file has not been verified, or the verification has expired.
    // So verify it exists, and cache the result
    fileCache.addPending(absolutePath, verifyFileExists(absolutePath));
    // wait for the file verification to finish
    const { exists } = await fileCache.get(absolutePath);
    if (!exists) {
        // Immediately remove the file from the cache
        fileCache.remove(absolutePath);
        throw new URIError(`Broken link to "${link}" in ${file.basename}.`);
    }
}
/**
 * Verifies that the specified file exists
 *
 * @param absolutePath - The absolute file path to check
 * @returns The timestamp when this verification expires
 */
async function verifyFileExists(absolutePath) {
    try {
        // Verify that the file exists (this will throw otherwise)
        await fs_1.promises.access(absolutePath);
        return {
            absolutePath,
            exists: true,
        };
    }
    catch (_) {
        return {
            absolutePath,
            exists: false,
        };
    }
}
/**
 * Rewrites a link to an MDX file
 *
 * NOTE: Next.js requires all links to be relative to the "pages" directory
 *
 * @param absolutePath - The absolute file path that the link points to
 * @param linkURL - The `absolutePath`, as a parsed "file://" URL
 * @param options - An options object
 * @returns The rewritten link, relative to the "pages" directory
 */
function rewriteLink(absolutePath, linkURL, options) {
    // Get the relative path from the "pages" directory
    let relativePath = path.sep + path.relative(options.pagesDir, absolutePath);
    if (path.basename(relativePath) === mdxIndex) {
        // Remove "index.mdx"
        relativePath = path.dirname(relativePath);
    }
    else {
        // Remove the ".mdx" extension
        relativePath = relativePath.slice(0, -mdxExtension.length);
    }
    // Convert the path to a relative URL
    const relativeURL = utils_1.toPosixPath(relativePath);
    // Append any querystring or hash from the original link
    return relativeURL + linkURL.search + linkURL.hash;
}
//# sourceMappingURL=link-to-mdx.js.map